<?php include("bee/compiler.php"); return; ?>
<script language="bee">
/************************************************************
 *	tree: Tree Object
 *
 *	The tree object takes an array of "parent pointers" in the form
 *	of {node}=>{parent}.  Three more arrays are generated: "child",
 *	"elder" and "younger".  Together with "parent", they form a tree
 *	reference structure (while the actual nodes are stored in an external
 *	hierarchy of some kind which is out of the scope of this object.
 *
 *	The order of nodes in the array indicates the rank of siblings, which is
 *	stored in the this%younger and this%elder variables.
 *
 *	arg%parent - the parent pointer array used to initialised the internal pointers
 *	arg%root - the index of the root node (default is the first node in the tree array)
 *
 *	Object variables:
 *	parent		the parent pointer
 *	child_{node}	the children array of {node}
 *
 *	root		the index of the root node
 *	//allnodes	the array of all node indexes
 *	current		the index of the currently traversing node
 *
 *	indent		the string used as tab in displaying the tree
 *	turbo		Whether to use lower-level coding to speed up the operation (default no)
 ************************************************************/

function tree
{
	// This constructor only initialise non-structural constructs.
	// All tree structure dependent ones are initialised in "reset"
	// to allow reconstructing the tree without calling the constructor.

	var this%turbo = "{arg%turbo|meanstrue}";

	// tree_init: Initialisation
	var this%function:reset = "tree_reset";
	var this%function:clear = "tree_clear";
	// PRIVATE: var this%function:setstruct = "tree_setstruct";

	// tree_stru: Structure
	var this%function:children = "tree_children";
	var this%function:siblings = "tree_siblings";
	var this%function:descendants = "tree_descendants";

	var this%function:getroot = "tree_getroot";
	//var this%function:getallnodes = "tree_getallnodes";

	// tree_disp: Dump
	// PRIVATE var this%function:dumpsubtree = "tree_dumpsubtree";
	var this%function:dump = "tree_dump";

	// tree_node: Node moving
	var this%function:current = "tree_current";

	var this%function:parent = "tree_parent";
	var this%function:child = "tree_child";	
	var this%function:elder = "tree_elder";	
	var this%function:younger = "tree_younger";	

	var this%function:root = "tree_root";
	var this%function:leaf = "tree_leaf";	
	var this%function:eldest = "tree_eldest";	
	var this%function:youngest = "tree_youngest";	

	// tree_take: Taking nodes
	var this%function:deletenode = "tree_deletenode";
	var this%function:isolatenode = "tree_isolatenode";

	// tree_adds: Adding nodes
	var this%function:addchild = "tree_addchild";
	var this%function:addelder = "tree_addelder";
	var this%function:addyounger = "tree_addyounger";
	// PRIVATE var this%function:insertelement = "tree_insertelement";

	// tree_port: Export and import trees
	var this%function:export = "tree_export";
	var this%function:import = "tree_import";

	// Data (After reset as it will clear everything)
	var this%indent = "&nbsp;&nbsp;&nbsp;&nbsp;";

	if ({arg%function:root|isset}) var param%tree_reset:root = "{arg%root}";
	this%tree_reset parent="(var)arg%parent";

	if ({arg%function:indent|isset}) var this%indent = "{arg%indent}";
}


include "common/tree_init.bs";
include "common/tree_stru.bs";
include "common/tree_disp.bs";
include "common/tree_node.bs";
include "common/tree_take.bs";
include "common/tree_adds.bs";
include "common/tree_port.bs";
</script>